{
 This unit implements OpenGL lights
}
unit GLLight;

interface
uses SysUtils,
     GL,
     GLU,
     GLUT,
     BaseTYpes;
type
    {
     This class is a wrapper for an OpenGL light.It encapsulates
     all the functionality of an OpenGL light source, in an
     easy-to-use container. It uses only GL_LIGHT0 to GL_LIGHT7.
     It does not query glGetIntegerv with GL_MAX_LIGHTS     
    }
    TLight=class(TObject)
                         private
                         fEnabled:boolean;
                         {
                          true - Light is enabled
                          false - Light is disabled
                         }
                         fIndex:glEnum;
                         {
                          Index of the light
                          ranges from GL_LIGHT0 to GL_LIGHT7
                         }
                         fAmbient:TColorf;
                         fDiffuse:TColorf;
                         fSpecular:TColorf;
                         {
                         Components of the light color
                         }
                         fPosition:TVector;
                         {
                         Position of the light source
                         }
                         fSpotDirection:TVector;
                         {
                         If a spotlight,denotes direction
                         }
                         fSpotExponent:Single;
                         fSpotCutoff:Single;
                         fConstantAtt:Single;
                         fLinearAtt:Single;
                         fQuadraticAtt:Single;
                         {
                         Quantities denoting the Attenuation and Intensity of the light
                         }
                         fInfinite:boolean;
                         {
                         Infinite source/Local source
                         }
                         protected
                         procedure SetAmbient(c:TColorF);
                         procedure SetDiffuse(c:TColorF);
                         procedure SetSpecular(c:TColorF);
                         procedure SetPosition(v:TVector);
                         procedure SetSpotDirection(d:TVector);
                         procedure SetSpotExponent(e:Single);
                         procedure SetSpotCutoff(c:Single);
                         procedure SetConstantAtt(a:Single);
                         procedure SetLinearAtt(a:Single);
                         procedure SetQuadraticAtt(a:Single);
                         procedure SetInfinite(i:Boolean);
                         {
                         Access methods for the respective properties
                         }
                         public
                         constructor Create(light:glEnum);
                         {
                         Create a light with the GL_LIGHT* parameter
                         }
                         procedure Enable;
                         {
                         Enables the light
                         }
                         procedure Disable;
                         {
                         Disables the light
                         }
                         property Ambient:TColorf
                                  read
                                  fAmbient write SetAmbient;
                         property Diffuse:TColorf
                                  read fDiffuse
                                  write SetDiffuse;
                         property Specular:TColorf
                                  read fSpecular
                                  write SetSpecular;
                         {
                         properties to access the components of the light
                         }
                         property Position:TVector
                                  read fPosition
                                  write SetPosition;
                         {
                         Set/Read the position of the light
                         }
                         property SpotDirection:TVector
                                  read fSpotDirection
                                  write SetSpotDirection;
                         {
                          Set/Read the direction of the light (if spotlight)
                          }
                         property SpotExponent:single
                                  read fSpotExponent
                                  write SetSpotExponent;
                         property SpotCutOff:Single
                                  read fSpotCutOff
                                  write SetSpotCutOff;
                         property ConstantAtt:Single
                                  read fConstantAtt
                                  write SetConstantAtt;
                         property LinearAtt:Single
                                  read fLinearAtt
                                  write SetLinearAtt;
                         property QuadraticAtt:Single
                                  read fQuadraticAtt
                                  write SetQuadraticAtt;
                         {
                         Set/Read the Intensity and Attenuation parameters of the light
                         }
                         property Infinite:boolean
                                  read fInfinite
                                  write SetInfinite;
                         {
                         Set/Read whether infinite or local light source
                         }
                         property Enabled:boolean
                                  read fEnabled;
                         {
                         Read only property shows whether light is enabled/disabled
                         }
    end;
implementation

constructor TLight.Create(light:glEnum);
begin
     inherited Create;
     fIndex:=Light;
     fAmbient:=MakeColorf(0,0,0,1);
     fDiffuse:=MakeColorf(1,1,1,1);
     fSpecular:=MakeColorf(1,1,1,1);
     fPosition:=MakeVector(0,0,1);
     fPosition[W]:=0;
     fInfinite:=true;
     fSpotDirection:=MakeVector(0,0,-1);
     fSpotExponent:=0;
     fSpotCutoff:=180;
     fConstantAtt:=1;
     fLinearAtt:=0;
     fQuadraticAtt:=0;
     fEnabled:=false;
end;
procedure TLight.Enable;
begin
     glEnable(fIndex);
     fEnabled:=true;
end;
procedure TLight.Disable;
begin
     glDisable(fIndex);
     fEnabled:=false;
end;
procedure TLight.SetAmbient(c:TColorf);
begin
     fAmbient:=c;
     glLightfv(fIndex,GL_AMBIENT,@fAmbient);
end;
procedure TLight.SetDiffuse(c:TColorf);
begin
     fDiffuse:=c;
     glLightfv(fIndex,GL_DIFFUSE,@fDiffuse);
end;
procedure TLight.SetSpecular(c:TColorf);
begin
     fSpecular:=c;
     glLightfv(fIndex,GL_SPECULAR,@fSpecular);
end;
procedure TLight.SetPosition(v:TVector);
begin
     if fInfinite then v[W]:=0 else v[W]:=1;
     fPosition:=v;
     glLightfv(fIndex,GL_POSITION,@fPosition);
end;
procedure TLight.SetSpotDirection(d:TVector);
begin
     fSpotDirection:=d;
     glLightfv(fIndex,GL_SPOT_DIRECTION,@fSpotDirection);
end;
procedure TLight.SetSpotExponent(e:single);
begin
     fSpotExponent:=e;
     glLightfv(fIndex,GL_SPOT_EXPONENT,@fSpotExponent);
end;
procedure TLight.SetSpotCutoff(c:single);
begin
     fSpotCutOff:=c;
     glLightfv(fIndex,GL_SPOT_CUTOFF,@fSpotCutOff);
end;
procedure TLight.SetConstantAtt(a:Single);
begin
     fConstantAtt:=a;
     glLightfv(fIndex,GL_CONSTANT_ATTENUATION,@fConstantAtt);
end;
procedure TLight.SetLinearAtt(a:single);
begin
     fLinearAtt:=a;
     glLightfv(fIndex,GL_LINEAR_ATTENUATION,@fLinearAtt);
end;
procedure TLight.SetQuadraticAtt(a:single);
begin
     fQuadraticAtt:=a;
     glLightfv(fIndex,GL_QUADRATIC_ATTENUATION,@QuadraticAtt);
end;
procedure TLight.SetInfinite(i:boolean);
begin
     fInfinite:=i;
     if i then
        fPosition[W]:=0
     else
         fPosition[W]:=1;
     glLightfv(fIndex,GL_POSITION,@fPosition);
end;

end.
